package com.mygdx.game.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.*;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.viewport.ExtendViewport;
import com.badlogic.gdx.utils.viewport.Viewport;
import com.mygdx.game.*;
import com.mygdx.game.Music.MusicController;
import com.mygdx.game.interrogation.Interrogation;
import com.mygdx.game.interrogation.ReaderDialogue;
import com.mygdx.game.interrogation.Sentences;
import com.mygdx.game.ui.UIElementCreator;
import com.mygdx.game.ui.UIStyle;
import com.mygdx.game.dialogue.DialogueBox;

import java.util.ArrayList;

/**
 * Shows the interrogation with all questions and answers for each person
 */
public class InterrogationScreen extends ScreenAdapter {

    private static Stage stage; // used to display user-Interface
    private static Viewport viewport; // display different resolutions
    private static Table mainTable;
    private static Texture backgroundTexture;
    private static Sprite backgroundSprite;
    private static SpriteBatch spriteBatch = new SpriteBatch();

    private static Image portraitActor;

    private static Button questionOneButton;
    private static Button questionTwoButton;
    private static DialogueBox dialogueBox = new DialogueBox();
    private static boolean isQuestionClicked = false;

    private static Interrogation interrogation;
    private static ArrayList<Sentences> sentences;

    private static int indexQuestions = 0;
    private static int indexAnswers = 0;
    private static boolean isDialogueOver = false;

    private static String personName;

    private Label info;


    /**
     * For this screen, it should be indicated who should now be interrogated
     *
     * @param dialogueFile the name of the dialogue file for example caro.json
     * @param name         the name of the interrogation person for example Caro Gerber
     */
    public InterrogationScreen(String dialogueFile, String name) {
        personName = name;
        interrogation = ReaderDialogue.getDialogueData(dialogueFile);
        sentences = interrogation.getSentences();

        String nameTrim = name.replaceAll(" ",""); // trim all whitespaces in string

        // get Images form the atlas
        TextureAtlas textureAtlas = Game.getGameInstance().assetManager.manager.get("personPortrait.atlas");
        portraitActor = new Image(textureAtlas.findRegion(nameTrim));
    }

    /**
     * Called when the screen should render itself
     * Change the question buttons for the next questions. Also checks if there are still question possibilities.
     * If there are no more question possibilities, the questions buttons will disappear and the final answer is written.
     * @param delta the time in seconds since the last render.
     */
    @Override
    public void render(float delta) {
        checkForUserInput();

        // clear screen
        Gdx.gl.glClearColor(1f, 1f, 1f, 1);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);

        // When there are no more question possibilities, deactivate the buttons
        if (indexQuestions == sentences.size()) {
            isQuestionClicked = false;
            deactivateQuestionButtons();
            info.setVisible(false);

            if (dialogueBox.isFinished() && isDialogueOver == false) {
                // wait some time before the final outro dialogue is displayed
                Wait.waitForMilliSeconds(3000);
                dialogueBox.animateText(interrogation.getOutro());
                isDialogueOver = true;
            }
        }

        // when a question button is clicked, the buttons need to display new questions
        if (isQuestionClicked) {
            isQuestionClicked = false;
            resetQuestionButtons();
        }

        stage.act();

        spriteBatch.begin();
        backgroundSprite.draw(spriteBatch);
        spriteBatch.end();

        stage.draw();
    }

    /**
     * Called when the application is resized
     *
     * @param width  the width of the application
     * @param height the height of the application
     */
    @Override
    public void resize(int width, int height) {
        viewport.update(width, height);
    }

    /**
     * Called when this screen becomes the current screen for the Game
     * Creates multiple UI Elements and layout them
     */
    @Override
    public void show() {

        createScreen();

        // index reset, otherwise an Out Of Range Exception for sentences will be thrown
        indexAnswers = 0;
        indexQuestions = 0;

        dialogueBox.animateText(interrogation.getIntro());

        // create UI Elements for this screen
        Label placeLabel = UIElementCreator.createLabel("Im Polizeirevier", UIStyle.Narration);
        Label nameLabel = UIElementCreator.createLabel("Befragt wird: " + personName, UIStyle.Window);
        info = UIElementCreator.createLabel("Wähle eine Frage aus:",UIStyle.Window);

        // Interrogation workflow
        questionOneButton = UIElementCreator.createTextButton(sentences.get(indexQuestions).getQuestion(), UIStyle.Default);
        questionOneButton.addListener(new ClickListener() {
            /**
             * Display answer for question choice number one in dialogueBox and add it also to the note window
             * @param event button is clicked
             * @param x
             * @param y
             */
            @Override
            public void clicked(InputEvent event, float x, float y) {
                String dialogueText = sentences.get(indexAnswers).getAnswer();
                dialogueBox.animateText(dialogueText);
                indexQuestions++;
                indexAnswers += 2;
                isQuestionClicked = true;

                MusicController.playButtonSound();

            }
        });

        questionTwoButton = UIElementCreator.createTextButton(sentences.get(++indexQuestions).getQuestion(), UIStyle.Default);
        questionTwoButton.addListener(new ClickListener() {
            /**
             * Display answer for question choice number two in dialogueBox and add it also to the note window
             * @param event button is clicked
             * @param x
             * @param y
             */
            @Override
            public void clicked(InputEvent event, float x, float y) {
                String dialogueText = sentences.get(++indexAnswers).getAnswer();
                dialogueBox.animateText(dialogueText);
                indexQuestions++;
                indexAnswers++;
                isQuestionClicked = true;

                MusicController.playButtonSound();
            }
        });



        Button backButton = UIElementCreator.createTextButton("Zurück", UIStyle.Comic);
        backButton.addListener(new ClickListener() {
            /**
             * Will change the screen to the MainMenuScreen
             * @param event button is clicked
             * @param x
             * @param y
             */
            @Override
            public void clicked(InputEvent event, float x, float y) {
                Game.getGameInstance().setScreen(new BackgroundScreen());

                MusicController.playButtonSound();
            }
        });



        // layout the UI elements
        mainTable.padTop(10);
        mainTable.add(placeLabel).top().left().pad(10f);
        mainTable.add(backButton).top().right().pad(10f).row();
        mainTable.add(nameLabel).left().padBottom(10f).padLeft(10f).row();
        mainTable.add(portraitActor).left().padBottom(10f).padLeft(10f).expand().row();
        mainTable.add(info).center().padBottom(10f).colspan(2).row();
        mainTable.add(questionOneButton).center().padBottom(10f).colspan(2).row();
        mainTable.add(questionTwoButton).center().padBottom(10f).colspan(2).row();
        mainTable.add(dialogueBox).center().pad(10f).colspan(2);

        Gdx.input.setInputProcessor(stage);
    }

    /**
     * Called when this screen is no longer the current screen
     */
    @Override
    public void hide() {
        super.hide();
    }

    /**
     * Called when this screen should release all resources
     */
    @Override
    public void dispose() {

        stage.dispose();
    }

    /**
     * Creates elements for the screen like stage, viewport and table
     */
    private static void createScreen() {
        viewport = new ExtendViewport(1280, 720);
        stage = new Stage(viewport);
        mainTable = new Table();
        mainTable.setFillParent(true);
        stage.addActor(mainTable);

        backgroundTexture = new Texture(Gdx.files.internal("ScreenBackgrounds/interrogationBackground.jpg"));
        backgroundSprite = new Sprite(backgroundTexture);
        backgroundSprite.setSize(Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
    }


    /**
     * Takes care of user input
     * Key: Escape ends game
     */
    public void checkForUserInput() {
        if (Gdx.input.isKeyJustPressed(Input.Keys.ESCAPE)) {
            Game.getGameInstance().setScreen(new BackgroundScreen());
        }
    }

    /**
     * Change question text for the question buttons
     */
    private static void resetQuestionButtons() {
        questionOneButton.reset();
        Label labelButtonOne = UIElementCreator.createLabel(sentences.get(indexQuestions++).getQuestion(), UIStyle.Comic);
        questionOneButton.add(labelButtonOne);

        questionTwoButton.reset();
        Label labelButtonTwo = UIElementCreator.createLabel(sentences.get(indexQuestions).getQuestion(), UIStyle.Comic);
        questionTwoButton.add(labelButtonTwo);
    }

    /**
     * Sets question buttons inactive and makes them invisible, this method should be called if there are no more question possibilities
     */
    private static void deactivateQuestionButtons() {
        questionOneButton.setDisabled(true);
        questionTwoButton.setDisabled(true);
        questionOneButton.setVisible(false);
        questionTwoButton.setVisible(false);
    }

}
