package com.mygdx.game.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.*;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.viewport.ExtendViewport;
import com.badlogic.gdx.utils.viewport.Viewport;
import com.mygdx.game.Game;
import com.mygdx.game.Music.MusicController;
import com.mygdx.game.ui.UIElementCreator;
import com.mygdx.game.ui.UIStyle;

public class OptionScreen extends ScreenAdapter {

    private static Stage stage; // used to display user-Interface
    private static Viewport viewport; // display different resolutions
    private static Table mainTable;

    private static Texture backgroundTexture;
    private static Sprite backgroundSprite;
    private static SpriteBatch spriteBatch = new SpriteBatch();
    private static Slider musicSlider;
    private static Slider buttonSoundSlider;

    /**
     * Called when the screen should render itself
     * Render takes care if music volume or button sound volume has been changed
     * @param delta the time in seconds since the last render.
     */
    @Override
    public void render(float delta) {
        // clear screen
        Gdx.gl.glClearColor(1f,1f,1f,1);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);

        if(musicSlider.isDragging())
        {
            MusicController.changeMusicVolume(musicSlider.getPercent());
        }

        if(buttonSoundSlider.isDragging())
        {
            MusicController.changeButtonSoundVolume(buttonSoundSlider.getPercent());
        }

        stage.act();

        spriteBatch.begin();
        backgroundSprite.draw(spriteBatch);
        spriteBatch.end();

        stage.draw();
    }

    /**
     * Called when the application is resized
     *
     * @param width  the width of the application
     * @param height the height of the application
     */
    @Override
    public void resize(int width, int height) {
        viewport.update(width,height);
    }

    /**
     * Called when this screen becomes the current screen for the Game
     * Creates multiple UI Elements and layout them
     */
    @Override
    public void show() {
        createScreen();

        // create UI elements
        Button backButton = UIElementCreator.createTextButton("Zurück", UIStyle.Comic);
        backButton.addListener(new ClickListener()
        {
            /**
             * Will change the screen to the MainMenuScreen
             * @param event button is clicked
             * @param x
             * @param y
             */
            @Override
            public void clicked(InputEvent event, float x, float y) {
                Game.getGameInstance().setScreen(new MainMenuScreen());

                MusicController.playButtonSound();
            }
        });

        // setup music options
        float musicVolume = MusicController.getMusicVolume();
        Label musicLabel = UIElementCreator.createLabel("Musik Volumen",UIStyle.Narration);
        musicSlider = UIElementCreator.createSlider();
        musicSlider.setVisualPercent(musicVolume);

        // setup button sounds options
        float buttonSoundVolume = MusicController.getButtonSoundVolume();
        Label soundLabel = UIElementCreator.createLabel("Button Sound", UIStyle.Narration);
        buttonSoundSlider = UIElementCreator.createSlider();
        buttonSoundSlider.setVisualPercent(buttonSoundVolume);

        // Layout UI elements
        VerticalGroup verticalGroup = new VerticalGroup();

        HorizontalGroup horizontalMusicGroup = new HorizontalGroup();
        horizontalMusicGroup.space(30f);
        horizontalMusicGroup.addActor(musicLabel);
        horizontalMusicGroup.addActor(musicSlider);

        HorizontalGroup horizontalSoundGroup = new HorizontalGroup();
        horizontalSoundGroup.space(30f);
        horizontalSoundGroup.addActor(soundLabel);
        horizontalSoundGroup.addActor(buttonSoundSlider);

        verticalGroup.addActor(horizontalMusicGroup);
        verticalGroup.addActor(horizontalSoundGroup);

        mainTable.add(backButton).top().right().pad(10f).row();
        mainTable.add(verticalGroup).center().expand();

        Gdx.input.setInputProcessor(stage);
    }

    /**
     * Called when this screen is no longer the current screen
     */
    @Override
    public void hide() {
        super.hide();
    }

    /**
     * Called when this screen should release all resources
     */
    @Override
    public void dispose() {
        stage.dispose();
    }

    /**
     * Creates elements for the screen like stage, viewport and table
     */
    private static void createScreen()
    {
        viewport = new ExtendViewport(1280, 720);
        stage = new Stage(viewport);
        mainTable = new Table();
        mainTable.setFillParent(true);
        stage.addActor(mainTable);

        backgroundTexture = new Texture(Gdx.files.internal("ScreenBackgrounds/policeRoomNight.jpg"));
        backgroundSprite = new Sprite(backgroundTexture);
        backgroundSprite.setSize(Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
    }
}
